﻿using Newtonsoft.Json;
using PSO2SERVER.Models;
using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace PSO2SERVER.Json
{
    public class JsonTest
    {
        public static void JsonReadTest()
        {
            // 测试数据文件路径
            string jsonFilePath1 = "data\\maps\\lobby\\data.json";
            string jsonFilePath2 = "data\\maps\\lobby\\events\\main_lobby_1\\all_events.json";
            string jsonFilePath3 = "data\\item_attrs.json";

            // 读取并反序列化 JSON 文件
            var map = DeserializeJson<MapData>(jsonFilePath1);
            var mapEvent = DeserializeJson<List<EventData>>(jsonFilePath2)?.FirstOrDefault();
            var attributes = DeserializeJson<ItemAttributesRootObject>(jsonFilePath3);

            // 输出 MapData 信息
            if (map != null)
            {
                Logger.Write($"map_object ID: {map.Mapdata.map_object.ID}");
                Logger.Write($"Zones[0].Name: {map.Zones[0].Name}");
                Logger.Write($"Zones[0].Is_special_zone: {map.Zones[0].Is_special_zone}");
            }

            // 输出 EventData 信息
            if (mapEvent != null)
            {
                Logger.Write($"map_event zone_id: {mapEvent.zone_id}");
                Logger.Write($"map_event is_active: {mapEvent.is_active}");
                Logger.Write($"map_event objName: {mapEvent.data.objName}");
            }

            // 输出 ItemAttributesPC 信息
            if (attributes != null)
            {
                Logger.Write($"PC unk1: {attributes.PC.Unk1}");
                ItemAttributesPC.LogWeapons(attributes.PC.Weapons);
                ItemAttributesPC.LogData17(attributes.PC.Data17);
            }
        }

        // 泛型方法：读取并反序列化 JSON 文件
        public static T DeserializeJson<T>(string filePath)
        {
            if (!File.Exists(filePath))
            {
                Logger.Write($"文件不存在: {filePath}");
                return default(T);
            }

            try
            {
                string json = File.ReadAllText(filePath);
                return JsonConvert.DeserializeObject<T>(json);
            }
            catch (Exception ex)
            {
                Logger.Write($"错误：读取或反序列化文件 {filePath} 时发生异常: {ex.Message}");
                return default(T);
            }
        }
    }
}
