﻿using Newtonsoft.Json;
using Org.BouncyCastle.Utilities;
using PSO2SERVER.Models;
using PSO2SERVER.Zone;
using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Runtime.Remoting.Messaging;
using System.Text;
using static PSO2SERVER.Models.PSOObject;

namespace PSO2SERVER.Protocol.Packets
{
    public class NPCSpawnPacket : Packet
    {
        [JsonProperty("object")]
        public ObjectHeader objHeader { get; set; } = new ObjectHeader();
        [JsonProperty("position")]
        public PSOLocation objPosition { get; set; } = new PSOLocation();
        public ushort unk1 { get; set; } = 0;
        /// Enemy name.
        [JsonProperty("name")]
        public string objName { get; set; } = string.Empty;//0x20
        public uint unk2 { get; set; } = 0;
        public byte[] unk3 { get; set; } = new byte[0x0C];
        public ushort unk4 { get; set; } = 0;
        public ushort unk5 { get; set; } = 0;
        public uint unk6 { get; set; } = 0;
        public uint unk7 { get; set; } = 0;
        public uint unk8 { get; set; } = 0;
        public uint unk9 { get; set; } = 0;
        public uint unk10 { get; set; } = 0;
        public uint unk11 { get; set; } = 0;
        public uint unk12 { get; set; } = 0;
        public string unk13 { get; set; } = string.Empty; //Ascii

        public NPCSpawnPacket(PSOObject obj)
        {
            objHeader = obj.Header;
            objPosition = obj.Position;
            objName = obj.Name;
            unk8 = 1101004800;
            unk12 = 1;
        }

        public NPCSpawnPacket()
        {
        }

        #region implemented abstract members of Packet

        public override byte[] Build()
        {
            PacketWriter pkt = new PacketWriter();
            pkt.WriteStruct(objHeader);
            pkt.WritePosition(objPosition);
            pkt.Write(unk1); // Padding?
            pkt.WriteFixedLengthASCII(objName, 0x20);
            pkt.Write(unk2); // Padding?
            pkt.Write(unk3); // Unknown, usually zero
            pkt.Write(unk4);
            pkt.Write(unk5);
            pkt.Write(unk6);
            pkt.Write(unk7);
            pkt.Write(unk8); // Always this 1101004800
            pkt.Write(unk9);
            pkt.Write(unk10);
            pkt.Write(unk11);
            pkt.Write(unk12);
            pkt.WriteAscii(unk13, 0x9FCD, 0xE7);

            return pkt.ToArray();
        }

        public override PacketHeader GetHeader()
        {
            return new PacketHeader(0x08, 0x0C, PacketFlags.PACKED);
        }

        #endregion
    }
}